﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using System.Windows;
using System.Windows.Controls;

namespace WpfBreakout.Bricks
{
    /* Could derive from Visual but would require a custom canvas to add visuals.
     * Canvas.Children accepts UIElements
     * 
     * 
     * Visual
     *      >UIElemnet
     *          >>FrameworkElement
     *      >ContainerVisual
     *          >>DrawingVisual
     *          >>HostVisual
     *      >Viweport3DVisual
     *      
     * UIElement can contain multiple visuals
     */

    public abstract class Brick : UIElement
    {
        // private Fields
        internal DrawingVisual _brick;
        internal const double _height = 10;
        internal const double _width = 25;

        // Properties
        public bool IsDistroyed = false;
        public abstract int Value { get; }
        private double Left { get { return (double)this.GetValue(Canvas.LeftProperty); } }
        private double Right { get { return Left + _width; } }
        private double Top { get { return (double)this.GetValue(Canvas.TopProperty); } }
        private double Bottom { get { return Top + _height; } }

        // overrides from Visual class
        protected override int VisualChildrenCount
        {
            get { return 1; }
        }
        protected override Visual GetVisualChild(int index)
        {
            if (index != 0)
            {
                throw new ArgumentOutOfRangeException();
            }

            return _brick;
        }


        // Methods
        public bool IsHit(Ball ball)
        {
            if (ball.Top > this.Bottom)
                return false;
            else if (ball.Bottom < this.Top)
                return false;
            else if (ball.Left > this.Right)
                return false;
            else if (ball.Right < this.Left)
                return false;

            this.IsDistroyed = true;
            return true;
        }
        public Collision HitLocation(Ball ball)
        {
            if (ball.Position.X > this.Right)
                return Collision.Right;
            else if (ball.Position.X < this.Left)
                return Collision.Left;
            else if (ball.Position.Y > this.Bottom)
                return Collision.Bottom;
            else if (ball.Position.Y < this.Top)
                return Collision.Top;

            else
                return Collision.Miss;

        }
        public virtual void Distroy(MainWindow window)
        {
            //var bonus = new BonusIncreaseLength(this, window);
            //window.GameCanvas.Children.Add(bonus);
        }
    }




    //public class Brick : Visual
    //{
    //    private DrawingVisual _brick;
    //    private const double _height = 10;
    //    private const double _width = 25;

    //    public Brick()
    //    {
    //        _brick = new DrawingVisual();
    //        DrawingContext context = _brick.RenderOpen();
    //        Rect rectangle = new Rect(0, 0, _width, _height);
    //        context.DrawRectangle(Brushes.Blue, new Pen(Brushes.Black, 1.0), rectangle);
    //        context.Close();
    //    }

    //    // overrides from Visual class
    //    protected override int VisualChildrenCount
    //    {
    //        get { return 1; }
    //    }
    //    protected override Visual GetVisualChild(int index)
    //    {
    //        if (index != 0)
    //        {
    //            throw new ArgumentOutOfRangeException();
    //        }

    //        return _brick;
    //    }

    //    // Properties
    //    private double Left { get { return (double)this.GetValue(Canvas.LeftProperty); } }
    //    private double Right { get { return Left + _width; } }
    //    private double Top { get { return (double)this.GetValue(Canvas.TopProperty); } } //Canvas.GetTop(this); } }// 
    //    private double Bottom { get { return Top + _height; } }


    //    public virtual int Value
    //    {
    //        get { return 100; }
    //    }
    //    public bool IsDistroyed = false;

    //    // Methods
    //    public void Distroy(MainWindow window)
    //    {
    //        var bonus = new BonusIncreaseLength(this, window);
    //        window.GameCanvas.Children.Add(bonus);
    //    }

    //    public bool HitBrick(Point p)
    //    {
    //        double left = (double)this.GetValue(Canvas.LeftProperty);
    //        double bottom = (double)this.GetValue(Canvas.BottomProperty);
    //        double top = bottom + 10;
    //        double right = left + 25;


    //        if (p.X < right && p.X > left)
    //        {
    //            if (p.Y < top && p.Y > bottom)
    //            {
    //                return true;
    //            }
    //            else
    //            {
    //                return false;
    //            }
    //        }
    //        else
    //        {
    //            return false;
    //        }
    //    }
    //    public bool HitBrick(Ball ball)
    //    {
    //        if (ball.Top > this.Bottom)
    //            return false;
    //        else if (ball.Bottom < this.Top)
    //            return false;
    //        else if (ball.Left > this.Right)
    //            return false;
    //        else if (ball.Right < this.Left)
    //            return false;




    //        this.IsDistroyed = true;
    //        return true;




    //        //if (ball.Position.X > Left && ball.Position.X < Right)
    //        //{
    //        //    if (ball.Position.Y > Top && ball.Position.Y < Bottom)
    //        //    {
    //        //        IsDistroyed = true;
    //        //        return true;
    //        //    }
    //        //}
    //        //return false;
    //    }

    //    public Collision HitLocation(Ball ball)
    //    {
    //        if (ball.Position.X > this.Right)
    //            return Collision.Right;
    //        else if (ball.Position.X < this.Left)
    //            return Collision.Left;
    //        else if (ball.Position.Y > this.Bottom)
    //            return Collision.Bottom;
    //        else if (ball.Position.Y < this.Top)
    //            return Collision.Top;

    //        else
    //            return Collision.Miss;

    //    }
    //}

}
