﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Threading;
using System.ComponentModel;
using WpfBreakout.Bricks;

namespace WpfBreakout
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged 
    {

        Ball ball = new Ball();
        bool InGame = false;
        public DispatcherTimer timer;

        // Private fields
        int lives;
        int score;
        TimeSpan time;
        int level;
        int _bricks;
        List<Brick> bricks;
        double PaddleTop
        {
            get { return Canvas.GetTop(paddle); }
        }
        double PaddleLeft
        {
            get { return Canvas.GetLeft(paddle); }
        }
        

        public int Lives
        {
            get
            {
                return lives;
            }
            set
            {
                lives = value;
                OnPropertyChanged("Lives");
                //infoPanel.Lives = lives;
            }
        }
        public TimeSpan Time
        {
            get
            {
                return time;
            }
            set
            {
                time = value;
                OnPropertyChanged("Time");
                //infoPanel.Time = time;
            }
        }
        public int Score 
        {
            get
            {
                return score;
            }
            set
            {
                score = value;
                OnPropertyChanged("Score");
                //infoPanel.Score = score;
            }
        }
        public int Level
        {
            get { return level; }
            set
            {
                level = value;
                OnPropertyChanged("Level");
                //infoPanel.Level = level;
            }
        }
        public int Bricks
        {
            get { return _bricks; }
            set
            {
                _bricks = value;
                OnPropertyChanged("Bricks");
                //infoPanel.Level = level;
            }
        }


        
        

        // Constructor
        public MainWindow()
        {
            InitializeComponent();
            MainGrid.DataContext = this;
            BallPath.DataContext = ball;
            infoPanel.DataContext = this;
            NewGame();
        }


        // Game Loop
        void timer_Tick(object sender, EventArgs e)
        {
            Time += TimeSpan.FromMilliseconds(20);

            ball.MoveBall();
            CheckWallCollisions();
            CheckPaddleCollisions();
            CheckBrickCollisions();
        }

        // Helper Methods
        public void NewGame()
        {
            Lives = 3;
            Time = TimeSpan.Zero;
            Score = 0;
            Level = 1;
            AddBricks();
            SetBall();
        }
        private void SetBall()
        {
            ball.Reset(new Point(PaddleLeft + paddle.Width / 2, PaddleTop - ball.Radius)); 
        }
        private void NextLevel()
        {
            Level++;
            SetBall();
            AddBricks();
            InGame = false;
        }

        private void CheckWallCollisions()
        {
            if (ball.Top > GameCanvas.ActualHeight)
                StopGame();
            else if (ball.Right > GameCanvas.ActualWidth)
                ball.SpeedX *= -1;
            else if (ball.Left < 0)
                ball.SpeedX *= -1;
            else if (ball.Top < 0)
                ball.SpeedY *= -1;
        }
        private void CheckPaddleCollisions()
        {
            if (ball.Direction.Y < 0)
                return;

            if (ball.Bottom < PaddleTop || ball.Top > PaddleTop + paddle.Height)
                return;

            double paddleCentre = PaddleLeft + paddle.Width / 2;
            double paddleRight = PaddleLeft + paddle.Width;
            double paddleRightEnd = paddleRight - paddle.Width / 4;
            double paddleLeftEnd = PaddleLeft + paddle.Width / 4;

            // Hits Right End
            if (ball.Position.X < paddleRight && ball.Position.X > paddleRightEnd)
            {
                ball.SpeedX = 10;
                ball.SpeedY *= -1;
            }
            // Hits Right of Centre
            else if (ball.Position.X < paddleRightEnd && ball.Position.X > paddleCentre)
            {
                ball.SpeedX = ball.SpeedX > 0 ? Ball.SPEED : -Ball.SPEED;
                ball.SpeedY *= -1;
            }
            // Hits Left of Centre
            else if (ball.Position.X < paddleCentre && ball.Position.X > paddleLeftEnd)
            {
                ball.SpeedX = ball.SpeedX < 0 ? -Ball.SPEED : Ball.SPEED;
                ball.SpeedY *= -1;
            }
            // Hits Left End
            else if (ball.Position.X < paddleLeftEnd && ball.Position.X > PaddleLeft)
            {
                ball.SpeedX = -10;
                ball.SpeedY *= -1;
            }



            //if (ball.Bottom >= PaddleTop && ball.Bottom <= (PaddleTop + 1))
            //{
            //    double paddleLeft = Canvas.GetLeft(paddle);
            //    double paddleCentre = paddleLeft + paddle.Width / 2;
            //    double paddleRight = paddleLeft + paddle.Width;
            //    double paddleRightEnd = paddleRight - paddle.Width / 4;
            //    double paddleLeftEnd = paddleLeft + paddle.Width / 4;


            //    // Hits Right End
            //    if (ball.Position.X < paddleRight && ball.Position.X > paddleRightEnd)
            //    {
            //        ball.SpeedX = 10;
            //        ball.SpeedY *= -1;
            //    }
            //    // Hits Right of Centre
            //    else if (ball.Position.X < paddleRightEnd && ball.Position.X > paddleCentre)
            //    {
            //        ball.SpeedX = ball.SpeedX > 0 ? Ball.SPEED : -Ball.SPEED;
            //        ball.SpeedY *= -1;
            //    }
            //    // Hits Left of Centre
            //    else if (ball.Position.X < paddleCentre && ball.Position.X > paddleLeftEnd)
            //    {
            //        ball.SpeedX = ball.SpeedX  < 0 ? -Ball.SPEED : Ball.SPEED;
            //        ball.SpeedY *= -1;
            //    }
            //    // Hits Left End
            //    else if (ball.Position.X < paddleLeftEnd && ball.Position.X > paddleLeft)
            //    {
            //        ball.SpeedX = -10;
            //        ball.SpeedY *= -1;
            //    }
            //}
        }
        private void CheckBrickCollisions()
        {
            List<Brick> hits = new List<Brick>();

            foreach (Brick b in bricks)
            {
                if (b.IsHit(ball))
                {
                    hits.Add(b);
                    b.Distroy(this);
                    GameCanvas.Children.Remove(b);

                    switch (b.HitLocation(ball))
                    {
                        case Collision.Left: ball.SpeedX *= -1;
                            break;
                        case Collision.Right: ball.SpeedX *= -1;
                            break;
                        case Collision.Top: ball.SpeedY *= -1;
                            break;
                        case Collision.Bottom: ball.SpeedY *= -1;
                            break;
                    }

                    Score += b.Value;
                }
            }

            foreach (Brick hit in hits)
            {
                if (hit.IsDistroyed)
                    bricks.Remove(hit);
            }

            Bricks = bricks.Count;

            if (Bricks == 0)
            {
                timer.Stop();
                MessageBox.Show("Winner");
                NextLevel();
            }
        }

        private void AddBricks()
        {
            bricks = new List<Brick>();
            Random rnd = new Random();

            for (int i = 0; i < 2 + Level; i++)
            {
                for (int j = 0; j < 18; j++)
                {
                    Brick b;
                    
                    if (rnd.NextDouble() > 0.8)
                        b = new SpecialBrick();
                    else
                        b = new NormalBrick();

                    double left = j * 25 + 25;
                    double top = i * 10 + 50;
                    b.SetValue(Canvas.LeftProperty, left);
                    b.SetValue(Canvas.TopProperty, top);
                    bricks.Add(b);
                    GameCanvas.Children.Add(b);
                    Bricks = bricks.Count;
                }
            }
        }




        // Events
        private void GameCanvas_MouseMove(object sender, MouseEventArgs e)
        {
            Point p = e.GetPosition(this);
            paddle.SetValue(Canvas.LeftProperty, p.X - paddle.Width / 2);

            if (!InGame)
            {
                ball.Reset(new Point(p.X, PaddleTop - ball.Radius));
            }
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            StartGame();
        }

        private void StartGame()
        {
            InGame = true;
            this.Cursor = Cursors.None;
            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(20);
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            StopGame();
        }

        private void StopGame()
        {
            if (timer != null)
                timer.Stop();
            timer = null;
            InGame = false;
            this.Cursor = Cursors.Arrow;
            SetBall();
            Lives -= 1;

            if (Lives == 0)
            {
                //MessageBox.Show("Looser");
                new StartScreen(this).ShowDialog();
            }
        }

        private void GameCanvas_MouseLeave(object sender, MouseEventArgs e)
        {
            this.Cursor = Cursors.Arrow;
        }

        private void GameCanvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (!InGame)
                StartGame();
        }

        


        public event PropertyChangedEventHandler PropertyChanged;
        
        protected void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;

            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        private void button3_Click(object sender, RoutedEventArgs e)
        {
            if (timer != null)
            {
                timer.IsEnabled = !timer.IsEnabled;
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //new StartScreen(this).ShowDialog();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            if (InGame)
            {
                switch (e.Key)
                {
                    case Key.Left: Canvas.SetLeft(paddle, PaddleLeft - 10);
                        break;
                    case Key.Right: Canvas.SetLeft(paddle, PaddleLeft + 10);
                        break;

                }
            }
        }

        private void button4_Click(object sender, RoutedEventArgs e)
        {
            StopGame();
            NewGame();
        }
    }
}
